# -*- coding: utf-8 -*-
#########################################################################
# File Name: filter_bkg.py
# Created on : 2022-12-17 16:43:57
# Author: JFF
# Last Modified: 2022-12-17 16:43:59
# Description:
# Usage:
# Input:
# Output:
#########################################################################
import os
import sys
import re
from pysam import FastxFile


def filter_bkg(negraw_bed, negraw_fa, exclude_bed):
    """
    Filter out the duplicate sequences and remove potential positive sequences from negative sequences.
    
    ## Parameters
    negraw_bed
        negraw_bed file from get_nullseq.
    negraw_fa
        negraw_fa file from get_nullseq.
    exclude_bed
        bed files contained potential positive regions. (e.g. peaks with P<0.05 from macs2).
    
    ## Returns
    out_neg_fa
        out_neg_fa can be used for trainning
    out_neg_bed
        out_neg_bed
    
    ## Notes
    Recommended at least setting ``exclude_bed`` as pos_bed to remove positive regions.
    
    """
    # out filename
    out_neg_bed = re.sub("\.negraw\.bed$", "", negraw_bed) + ".neg.bed"
    out_neg_fa = re.sub("\.negraw\.fa$", "", negraw_fa) + ".neg.fa"
    #used chr
    used_chrset = ["chr" + str(i) for i in list(range(1, 24))]
    pos_set = set([])
    name_set = set([])
    # find negraw bed intersect with exclude_bed
    with os.popen("awk 'BEGIN{OFS=\"\t\"}{print $0,$1\"_\"$2\"_\"$3\"_neg_\"NR}' %a|bedtools intersect -v -a - -b %s -wo " %
                  (negraw_bed, exclude_bed)) as f1:
        with open(out_neg_bed, 'w') as ff1:
            for i in f1:
                i = i.strip().split("\t")
                if i[0] in used_chrset:
                    name = i[3]  # chr3_159811111_159811310_neg_11
                    pos = "_".join(i[:3])  # chr3_159811111_159811310
                    if pos not in pos_set:  # 用位置判断是否重复
                        pos_set.add(pos)
                        name_set.add(name)
                        ff1.write("\t".join(i[:3]) + "\n")
    # write filtered fa to new file
    with FastxFile(negraw_fa) as f1:
        with open(out_neg_fa, 'w') as ff2:
            for i in f1:
                name = i.name
                seq = i.sequence
                if re.sub("^>", "", name) in name_set:
                    ff2.write(">" + name + "\n")
                    ff2.write(seq + "\n")
    return out_neg_fa, out_neg_bed


if __name__ == '__main__':
    filter_bkg("../Utilities_pipeline/example.output_eQTac/test.positive.negraw.bed", 
               "../Utilities_pipeline/example.output_eQTac/test.positive.negraw.fa",
               "../Utilities_pipeline/test_data/test.exclude.bed")



# -*- coding: utf-8 -*-
#########################################################################
# File Name: eQTac_correlation.py
# Created on : 2022-12-19 21:39:53
# Author: JFF
# Last Modified: 2023-09-22 21:59:02
# Description:
# Usage:
# Input:
# Output:
#########################################################################
import os
import sys
import re
from pybedtools import BedTool
from scipy.stats import linregress
import numpy as np
import pandas as pd


def st_linear_reg(exp, score):  # 这个速度更快
    # select = ~np.isnan(d_score[pre])  #得分不为nan（基因型不缺失）的pre
    #slope, intercept, r_value, p, std_err = stats.linregress(score[select], exp[select])
    slope, intercept, r_value, p, std_err = linregress(score, exp)
    return (slope, std_err, p, intercept, r_value)  # beta,se,p


def extract_data(PRE_scorefile, exp_file, upstream_bp=1000000, downstream_bp=1000000):
    score_data = pd.read_csv(PRE_scorefile, sep="\t", header=0, index_col=3).T
    exp_data = pd.read_csv(exp_file, sep="\t", header=0, index_col=3).T
    # 提取基因cis区域（1MB）
    gene_range = exp_data.loc[["#chr", "start"], :].T
    d_gene_info = {i: list(gene_range.loc[i, :]) for i in gene_range.T}  # chr,tss,保存gene位置信息，用于后面输出
    gene_range.loc[:, "end"] = gene_range.loc[:, "start"] + downstream_bp
    gene_range.loc[:, "start"] = gene_range.loc[:, "start"] - upstream_bp
    gene_range.loc[:, "ENSG"] = gene_range.index
    gene_range.loc[gene_range.loc[:, "start"] < 0, "start"] = 0
    gene_range = BedTool(gene_range.values.tolist())  # chr,start,end, ensg
    # 提取pre区域
    pre_range = score_data.loc[["chr", "start", "end", "snp_count", "snps"], :].T
    d_pre_info = {i: list(pre_range.loc[i, :]) for i in pre_range.T}  # chr,start,end,snp_count,snps,保存pre位置信息，用于后面输出
    pre_range.loc[:, "pre"] = pre_range.index
    pre_range = BedTool(pre_range.values.tolist())  # chr,start,end,snp_count,snps,pre
    # 获取每个基因对应的SNP
    d_cis = {}
    for i in gene_range.intersect(pre_range, wo="-wo"):
        d_cis.setdefault(i[3], []).append(i[9])  # ensg:[pre,pre]
    score_data = score_data.iloc[5:, :]
    exp_data = exp_data.iloc[5:, :]
    # 统一样本顺序
    samplelist = list(set(score_data.index) & set(exp_data.index))
    exp_data = exp_data.loc[samplelist, :]
    score_data = score_data.loc[samplelist, :]
    return score_data, exp_data, d_gene_info, d_cis, d_pre_info


def calculate_correlation(score_data, exp_data, d_gene_info, d_cis, d_pre_info, ff):
    # 基因表达和PRE信息存成字典
    # snp score
    d_score = {}
    for pre in score_data.columns:
        res = np.array(score_data.loc[:, pre]).astype(float)
        d_score[pre] = res  # pre:[score,score,]
    # exp data
    d_exp = {}
    for ensg in exp_data.columns:
        d_exp[ensg] = np.array(exp_data.loc[:, ensg]).astype(float)  # ensg:[exp,exp,]
    # 计算回归方程并输出结果
    ff.write("ENSG\tchr\tTSS\tPRE\tpre_start\tpre_end\tbeta\tse\tp\tpre_snps\n")
    for ensg in d_cis:
        ch, tss = d_gene_info[ensg][:]
        for pre in d_cis[ensg]:
            # if pre in d_score:  #有些SNP两个型没有差异，没存到字典中
            score = d_score[pre]
            exp = d_exp[ensg]
            beta, se, p, intercept, r_value = st_linear_reg(exp, score)
            pre_start, pre_end, snp_count, snps = d_pre_info[pre][1:]
            ff.write("\t".join(
                [ensg, ch, "%d" %
                 tss, pre, "%d" %
                 pre_start, "%d" %
                 pre_end, "%.5g" %
                 beta, "%.5g" %
                 se, "%.5g" % p, snps]) + "\n")
        #print(ensg, "is done")


def eQTac_correlation(PRE_scorefile, exp_file, result_file, upstream_bp=1000000, downstream_bp=1000000):
    score_data, exp_data, d_gene_info, d_cis, d_pre_info = extract_data(PRE_scorefile, exp_file, upstream_bp, downstream_bp)
    with open(result_file, 'w') as ff:
        calculate_correlation(score_data, exp_data, d_gene_info, d_cis, d_pre_info, ff)


if __name__ == "__main__":
    eQTac_correlation("../Utilities_pipeline/example.output_eQTac/test.geno.vcf.gz.PRE_score",
                      "../Utilities_pipeline/test_data/test.exp_residual",
                      "../Utilities_pipeline/example.output_eQTac/test.geno.vcf.gz.PRE_score.eQTac_result")



# -*- coding: utf-8 -*-
#########################################################################
# File Name: geno2score.py
# Created on : 2022-12-19 20:30:37
# Author: JFF
# Last Modified: 2024-02-20 13:56:58
# Description:
# Usage:
# Input:
# Output:
#########################################################################
import os
import sys
import re
import numpy as np
import gzip


def geno2score(vcf_gz, pred_out, ld_info,normlize=True,ld_max=0.3,distance_min=10):
    PRE_scorefile = vcf_gz + ".PRE_score"
    # 增强子上的SNP
    d_pre = {}
    with open(ld_info) as f1:
        next(f1)
        for i in f1:
            i = i.strip().split("\t")
            if float(i[6]) <= ld_max and int(i[9]) > distance_min:  # enhancer上SNP之间最小的ld需要<0.3，SNP之间最小距离大于10bp
                enhancer = "_".join(i[:3])
                snp_set = i[11].split(",")
                d_pre[enhancer] = snp_set  # enhancer:set([snp,snp])
    # SNP得分
    d_score = {}
    with open(pred_out) as f2:
        for i in f2:
            i = i.strip().split("\t")
            snp = i[0].split(":")[0]
            allele = i[0].split(":")[-1]
            score = float(i[1])
            d_score.setdefault(snp, {})[allele] = score  # snp:{allele1:score,}

    # 将基因型转化为得分
    d_geno = {}
    miss_snp_set = set([])
    with gzip.open(vcf_gz, 'rt') as f3:
        for i in f3:
            i = i.strip().split("\t")
            if not i[0].startswith("##"):  # 注释行
                if i[0].startswith("#"):  # 标题行
                    header = i[9:]  # 保存标题行的样本顺序
                else:
                    snp = i[2]
                    if snp in d_score:  # snp有预测得分
                        geno_list = i[9:]
                        for j in range(len(header)):
                            geno = geno_list[j]  # 0/1,
                            alleles = re.sub("1", i[4], re.sub("0", i[3], geno)).split("/")  # 转成[A1,A2]
                            if "." in alleles:  # 基因型缺失，直接用“.”代替
                                score = "NA"
                                miss_snp_set.add(snp)  # 有缺失值的SNP
                            else:  # 基因型不缺失，计算得分
                                score = d_score[snp][alleles[0]] + d_score[snp][alleles[1]] # 两个型得分求和
                            d_geno.setdefault(snp, {})[header[j]] = score  # snp:{OA-1:score,}
    # 填补缺失值（人群中的该SNP得分的均值）
    for snp in miss_snp_set:
        mean_score = np.mean([i for i in d_geno[snp].values() if i != "NA"])
        for sample in d_geno[snp]:
            if d_geno[snp][sample] == "NA":
                d_geno[snp][sample] = mean_score
    # 输出每个人每个enhancer的得分
    with open(PRE_scorefile, 'w') as ff:
        ff.write("\t".join(["chr", "start", "end", "PRE", "snp_count", 'snps'] + header) + "\n")
        for enhancer in d_pre:
            ch, start, end = enhancer.split("_")
            snp_set = [snp for snp in d_pre[enhancer] if snp in d_geno]
            snp_count = len(snp_set)
            score_list = [sum([d_geno[snp][sample] for snp in snp_set]) for sample in header]
            if normlize:
                score_list_mean=np.mean(score_list)
                score_list_std=np.std(score_list)
                score_list=(np.array(score_list)-score_list_mean)/score_list_std
            score_list = ["%.6g" % i for i in score_list]
            snps = ",".join(snp_set)
            ff.write("\t".join([ch, start, end, enhancer, "%d" % snp_count, snps] + score_list) + "\n")
    return PRE_scorefile


if __name__ == "__main__":
    geno2score(
        "../Utilities_pipeline/example.output_eQTac/test.geno.vcf.gz",
        "../Utilities_pipeline/example.output_eQTac/test.geno.snplist.bed--test.pre.bed.pre_snplist.ld_info.snplist.bed.mutate.pred_out",
        "../Utilities_pipeline/example.output_eQTac/test.geno.snplist.bed--test.pre.bed.pre_snplist.ld_info"
    )



# -*- coding: utf-8 -*-
#########################################################################
# File Name: generate_snp_dict.py
# Created on : 2022-12-21 15:43:12
# Author: JFF
# Last Modified: 2022-12-21 15:43:14
# Description:
# Usage:
# Input:
# Output:
#########################################################################
import os
import sys
import re


def generate_snp_dict(bim_file):
    """
    Generate a dict from a plink .bim file. Used in generate_PRE and generate_mut_fa.
    
    ## Parameters
    bim_file: bim file path
        chr\tsnp\tcmol\tpos\tA1\tA2\n
        chr1\\trs202152658\\t0\\t751343\\tA\\tT
    
    ## Returns
    d_snp: dict
        {snp:[chr,pos,a1,a2],...}
    """
    d_snp = {}
    with open(bim_file, 'r') as f1:
        for i in f1:
            i = i.strip().split("\t")
            d_snp[i[1]] = [i[0], i[3], i[4], i[5]]  #snp:[chr,pos,a1,a2]
    return d_snp



# -*- coding: utf-8 -*-
#########################################################################
# File Name: generate_mut_fa.py
# Created on : 2022-12-19 17:02:20
# Author: JFF
# Last Modified: 2022-12-19 17:02:21
# Description:
# Usage:
# Input:
# Output:
#########################################################################
import os
import sys
import re
import time
import math


def generate_mut_fa(ld_info, genome_fasta, d_snp):
    # Generate snp bed file
    snp_bed = ld_info + ".snplist.bed"
    snpset = set([])
    with open(ld_info) as f2:
        next(f2)
        for i in f2:
            i = set(i.strip().split("\t")[11].split(","))
            snpset = snpset | i
    with open(snp_bed, 'w') as ff1:
        for snp in snpset:
            ch, pos, a1, a2 = d_snp[snp]
            if len(a1) == 1 and len(a2) == 1:  # only SNPs
                ff1.write("\t".join([ch, str(int(pos) - 10), str(int(pos) + 9), snp, a1, a2]) + "\n")
    # Generate fa file
    snp_fa = snp_bed + ".fa"
    os.system("bedtools getfasta -fi %(genome_fasta)s -bed %(snp_bed)s -name |seqkit seq -u > %(snp_fa)s" % {
        "genome_fasta": genome_fasta,
        "snp_bed": snp_bed,
        "snp_fa": snp_fa
    })
    # Generate mutate fa file
    d = {}
    with open(snp_bed) as f1:
        for i in f1:
            i = i.strip().split("\t")
            d.setdefault(i[3], {})[i[4]] = i[5]
            d.setdefault(i[3], {})[i[5]] = i[4]  # rsxxx:{A:T,T:A}
    mutate_fa = re.sub("\.fa$", "", snp_fa) + ".mutate.fa"
    with open(snp_fa) as f1:
        with open(mutate_fa, 'w') as ff:
            l_save = []
            n = 0
            for i in f1:
                i = i.strip()
                n += 1
                if n % 2 == 1:
                    l_save.append([])
                    n_pair = n // 2
                    l_save[n_pair].append(i)
                elif n % 2 == 0:
                    n_pair = n // 2 - 1
                    l_save[n_pair].append(i)
            for i in l_save:
                rs = re.sub("^>", "", i[0].split("::")[0])
                raw_allele = i[1][9]
                mutate_allele = d[rs][raw_allele]
                raw_title = i[0] + ":" + raw_allele + "_" + mutate_allele + ":" + raw_allele
                mutate_title = i[0] + ":" + raw_allele + "_" + mutate_allele + ":" + mutate_allele
                raw_seq = i[1]
                mutate_seq = i[1][:9] + mutate_allele + i[1][10:]
                ff.write("\n".join([raw_title, raw_seq]) + "\n")
                ff.write("\n".join([mutate_title, mutate_seq]) + "\n")
    return mutate_fa


# -*- coding: utf-8 -*-
#########################################################################
# File Name: control_FDR.py
# Created on : 2022-12-20 19:51:25
# Author: JFF
# Last Modified: 2023-04-18 17:28:48
# Description:
# Usage:
# Input:
# Output:
#########################################################################
import os
import sys
import re
import time
import pandas as pd
import numpy as np


def control_FDR(PRE_scorefile, permute_file, output_file):
    permute_plist = [float(i.strip()) for i in open(permute_file)]
    # observe_data 处理
    observe_data = pd.read_table(PRE_scorefile, header=0, index_col=None, sep="\t")
    observe_data = observe_data.sort_values(by="p")
    observe_data.loc[:, "FDR"] = 1
    observe_plist = list(observe_data.loc[:, "p"])
    # permute plist处理
    n_fold = len(permute_plist) / len(observe_plist)  # 背景集是真实值数量的多少倍
    permute_plist = np.array(permute_plist)
    # 计算FDR及阈值
    fdr_list = []
    for i in range(len(observe_plist)):
        n_lower = np.sum(permute_plist < observe_plist[i]) / n_fold
        fdr = n_lower / (i + 1)
        fdr_list.append(fdr)
    # FDR p值排序在后面的，FDR应大于等于前面的FDR值
    fdr_last = -1
    for i in range(len(fdr_list)):
        fdr_list[i] = max(fdr_list[i], fdr_last)
        fdr_last = fdr_list[i]
    observe_data.loc[:, "FDR"] = [i if i < 1 else 1 for i in fdr_list]
    observe_data = observe_data.sort_values(by="p")
    # 保存文件
    observe_data.to_csv(output_file, sep="\t", header=True, index=False)


if __name__ == "__main__":
    control_FDR(
        "../Utilities_pipeline/example.output_eQTac/test.geno.vcf.gz.PRE_score.eQTac_result",
        "../Utilities_pipeline/example.output_eQTac/test.geno.vcf.gz.PRE_score.eQTac_result.permutation_plist",
        "../Utilities_pipeline/example.output_eQTac/test.geno.vcf.gz.PRE_score.eQTac_result.FDR.txt"
    )



import setuptools

with open("README.md", "r") as f1:
    long_description = f1.read()

setuptools.setup(
    name="eQTac",
    version="1.0.15",
    author="Jiang Feng",
    author_email="1594032292@qq.com",
    description="The eQTac method.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/JFF1594032292/eQTac",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: BSD License",
    ],
    python_requires='==3.8',
    install_requires=[
        'numpy == 1.22.4',
        'pandas == 1.4.3',
        'pybedtools == 0.8.2',
        'pysam == 0.16.0.1',
        'rpy2 == 3.5.11',
        'scipy == 1.8.1'],
)



# -*- coding: utf-8 -*-
#########################################################################
# File Name: eQTac_permutation.py
# Created on : 2022-12-20 15:58:46
# Author: JFF
# Last Modified: 2023-09-22 22:00:21
# Description:
# Usage:
# Input:
# Output:
#########################################################################
import os
import sys
import re
import numpy as np
from .eQTac_correlation import extract_data, st_linear_reg


def eQTac_permutation(PRE_scorefile, exp_file, permute_count, result_file, upstream_bp=1000000, downstream_bp=1000000):
    score_data, exp_data, d_gene_info, d_cis, d_pre_info = extract_data(PRE_scorefile, exp_file, upstream_bp, downstream_bp)
    with open(result_file, 'w') as ff:
        for seed in range(permute_count):
            exp_data_copy = exp_data.copy()
            # 随机置换表达值
            rng = np.random.default_rng(seed)
            exp_label = list(exp_data.index)
            rng.shuffle(exp_label)
            exp_data_copy = exp_data_copy.loc[exp_label, :]
            # 基因表达和PRE信息存成字典
            # snp score
            d_score = {}
            for pre in score_data.columns:
                res = np.array(score_data.loc[:, pre]).astype(float)
                d_score[pre] = res  # pre:[score,score,]
            # exp data
            d_exp = {}
            for ensg in exp_data_copy.columns:
                d_exp[ensg] = np.array(exp_data_copy.loc[:, ensg]).astype(float)  # ensg:[exp,exp,]
            # 计算回归方程
            for ensg in d_cis:
                for pre in d_cis[ensg]:
                    # if pre in d_score:  #有些SNP两个型没有差异，没存到字典中
                    score = d_score[pre]
                    exp = d_exp[ensg]
                    beta, se, p, intercept, r_value = st_linear_reg(exp, score)
                    ff.write("%.5g" % p + "\n")
            print("Permutation", seed + 1, "finished")


if __name__ == "__main__":
    eQTac_permutation(
        "../Utilities_pipeline/example.output_eQTac/test.geno.vcf.gz.PRE_score",
        "../Utilities_pipeline/test_data/test.exp_residual", 100,
        "../Utilities_pipeline/example.output_eQTac/test.geno.vcf.gz.PRE_score.eQTac_result.permutation_plist")



# -*- coding: utf-8 -*-
#########################################################################
# File Name: get_nullseq.py
# Created on : 2022-12-17 16:06:59
# Author: JFF
# Last Modified: 2023-07-28 11:49:29
# Description:
# Usage:
# Input:
# Output:
#########################################################################
import os
import sys
import re
import rpy2.robjects as robjects
from shutil import copyfile


def get_nullseq(pos_bed_path, out_folder, xfold):
    """
    Generates null sequences (negative set) with matching repeat and GC content as the input bed file for positive set regions.\n
    A wrapper of genNullSeqs from gkmSVM R package.

    ## Parameters
    pos_bed_path
        positive training set bed file. e.g. ATAC peak bed file.
    out_folder
        output directory

    ## Returns
    pos_fa
        positive sequence .fa file
    pos_bed_newpath
        positive sequence .bed file. (copy to output directory)
    negraw_fa
        raw negative sequence .fa file.
    negraw_bed
        raw negative sequence .bed file.


    ## Notes
    The output ``negraw_fa`` and ``negraw_bed`` file not remove the duplicate sequences, and raw negative sequences may also contain potential positive sequences.(e.g. peaks with P<0.05 but q>0.05)\n
    The ``negraw_fa`` and ``negraw_bed`` showed filtered by ``filter_bkg`` function.

    """
    pos_bed = os.path.basename(pos_bed_path)
    prefix = re.sub("\.bed$", "", pos_bed)
    # make dir
    if not os.path.exists(out_folder):
        os.mkdir(out_folder)
    # copy pos_bed to output folder & add a "pos" suffix
    pos_bed_newpath = out_folder + "/" + prefix + ".pos.bed"
    copyfile(pos_bed_path, pos_bed_newpath)
    # gkmSVM genNullSeqs
    pos_fa = out_folder + "/" + prefix + ".pos.fa"
    negraw_bed = out_folder + "/" + prefix + ".negraw.bed"
    negraw_fa = out_folder + "/" + prefix + ".negraw.fa"

    d = dict(pos_bed_newpath=pos_bed_newpath, pos_fa=pos_fa, negraw_bed=negraw_bed, negraw_fa=negraw_fa, xfold=xfold)
    robjects.r(
        "library(gkmSVM);genNullSeqs('%(pos_bed_newpath)s',nMaxTrials=20,xfold=%(xfold)f,GC_match_tol=0.05,repeat_match_tol=0.05,length_match_tol=0.05,batchsize=500000,genomeVersion='hg19', outputPosFastaFN='%(pos_fa)s', outputBedFN='%(negraw_bed)s', outputNegFastaFN='%(negraw_fa)s')"
        % d)
    return pos_fa, pos_bed_newpath, negraw_fa, negraw_bed


if __name__ == '__main__':
    get_nullseq("../Utilities_pipeline/test_data/test.positive.bed", ".", xfold=2.5)



# -*- coding: utf-8 -*-
#########################################################################
# File Name: generate_PRE.py
# Created on : 2022-12-19 10:19:00
# Author: JFF
# Last Modified: 2024-02-20 13:57:17
# Description:
# Usage:
# Input:
# Output:
#########################################################################
import os
import sys
import re
import time
import math
import numpy as np
from itertools import combinations
from shutil import copyfile


def generate_PRE(geno_prefix, pre_bed, snp_list, d_snp, outfolder, ld_max=0.3, distance_min=10):
    # make dir
    if not os.path.exists(outfolder):
        os.mkdir(outfolder)
    # get max pre length
    pre_length_set = set([])
    with open(pre_bed) as f1:
        for i in f1:
            i = i.strip().split("\t")
            pre_length_set.add(abs(int(i[2]) - int(i[1])))
    pre_max_length_kb = math.ceil(max(pre_length_set) / 1000)
    # generate snp bed
    snp_bed = outfolder + "/" + os.path.basename(snp_list) + ".bed"
    with open(snp_list) as f1:
        with open(snp_bed, 'w') as ff1:
            for i in f1:
                snp = i.strip()
                ch, pos, a1, a2 = d_snp[snp]
                if len(a1) == 1 and len(a2) == 1:
                    ff1.write("\t".join([ch, pos, pos, snp]) + "\n")
    # str format
    d_str = {
        "pre_bed": pre_bed,
        "pre_bed_name": os.path.basename(pre_bed),
        "snp_bed": snp_bed,
        "snp_bed_name": os.path.basename(snp_bed),
        "geno_prefix": geno_prefix,
        "outfolder": outfolder,
        "pre_max_length_kb": pre_max_length_kb
    }
    # 提取pre上的SNP，计算ld矩阵
    os.system(
        "bedtools intersect -a %(snp_bed)s -b %(pre_bed)s  |cut -f4|sort -u > %(outfolder)s/%(snp_bed_name)s--%(pre_bed_name)s.pre_snplist"
        % d_str)
    os.system(
        "plink --allow-no-sex --bfile %(geno_prefix)s --extract %(outfolder)s/%(snp_bed_name)s--%(pre_bed_name)s.pre_snplist --r2  --ld-window-r2 0 --ld-window-kb %(pre_max_length_kb)d --ld-window 999999 --out %(outfolder)s/%(snp_bed_name)s--%(pre_bed_name)s.pre_snplist"
        % d_str)
    # 提取ld信息
    d_ld = {}
    with open("%(outfolder)s/%(snp_bed_name)s--%(pre_bed_name)s.pre_snplist.ld" % d_str) as f1:
        next(f1)
        for i in f1:
            i = i.strip().split()
            snp1 = i[2]
            snp2 = i[5]
            r2 = float(i[6])
            d_ld.setdefault(snp1, {})[snp2] = r2
            # snp2:{snp1:r2},snp1:{snp2:r2}
            d_ld.setdefault(snp2, {})[snp1] = r2
    # 获取SNP的位置
    d_pos = {}
    with open(snp_bed) as f1:
        for i in f1:
            i = i.strip().split("\t")
            d_pos[i[3]] = int(i[2])  # snp:pos
    # 计算平均r2并输出
    d_peak = {}
    with os.popen("bedtools intersect -a %(snp_bed)s -b %(pre_bed)s -wa -wb" % d_str) as f1:
        for i in f1:
            i = i.strip().split("\t")
            peak = i[4:8]
            if len(peak) == 3:  # 没有名字的话加上名字
                peak = peak + ["PRE_" + "_".join(peak[:3])]
            peak = tuple(peak)
            snp = i[3]
            d_peak.setdefault(peak, set([])).add(snp)  # peak:set([snp,snp,])

    with open("%(outfolder)s/%(snp_bed_name)s--%(pre_bed_name)s.pre_snplist.ld_info" % d_str, 'w') as ff:
        ff.write("chr\tstart\tend\tPRE_region\tsnp_count\tmean_r2\tmin_r2\tr2\tmean_distance\tmin_distance\tdistance\tsnp_set\n")
        for peak in d_peak:
            snp_set = d_peak[peak]
            poss = [d_pos[j] for j in snp_set]
            distance_list = [abs(j[0] - j[1]) for j in combinations(poss, 2)]
            r2_list = []
            for snp_pair in combinations(snp_set, 2):
                # print(snp_pair[0], snp_pair[1])
                try:
                    r2 = d_ld[snp_pair[0]][snp_pair[1]]
                    r2_list.append(r2)
                except:
                    # print(snp_pair[0], snp_pair[1], "no ld information!")
                    continue
            # print(snp_set)
            # print(r2_list)
            if len(snp_set) > 1:
                r2_mean = np.mean(r2_list)
                r2_min = np.min(r2_list)
                min_dis = min(distance_list)
                mean_dis = np.mean(distance_list)
                if r2_min <= ld_max and min_dis > distance_min:
                    ff.write("\t".join(peak) + "\t" + "%d" % len(snp_set) + "\t" + "%.4g" % r2_mean + "\t" + "%.4g" % r2_min + "\t" +
                             ",".join(["%.3g" % i for i in r2_list]) + "\t" + "%.4g" % mean_dis + "\t" + "%.4g" % min_dis + "\t" +
                             ",".join(["%.3g" % i for i in distance_list]) + "\t" + ",".join(snp_set) + "\n")

    return "%(outfolder)s/%(snp_bed_name)s--%(pre_bed_name)s.pre_snplist.ld_info" % d_str
